@@ -1,6 +1,7 @@
 from .tools import MISSING
+from .errors import InvalidLength, OutOfValidRange, WrongType
 
-from discord import Emoji, emoji
+from discord import Emoji
 from discord.errors import InvalidArgument
 
 import inspect
@@ -20,10 +21,6 @@ class SelectOption():
         An additional description of the option, max 50 characters
     emoji : :class:`discord.Emoji` | :class:`str`, optional
         Emoji appearing before the label; default MISSING
-
-    Raises
-    ------
-    :raises: :class:`discord.InvalidArgument`: A passed argument was invalid
     """
     def __init__(self, value, label="\u200b", description=MISSING, emoji=MISSING) -> None:
         """
@@ -42,7 +39,8 @@ class SelectOption():
             self.description = description
         if emoji is not MISSING:
             self.emoji = emoji
-        
+    def __repr__(self) -> str:
+        return "<SelectOption(" + self.content + ")>"
 
     @property
     def content(self) -> str:
@@ -51,7 +49,7 @@ class SelectOption():
 
         :type: :class:`str`
         """
-        return (self.emoji + " " if self.emoji is not None else "") + (self.label or '')
+        return (self.emoji + " ") if self.emoji is not None else "" + (self.label or '')
     
     @property
     def label(self) -> str:
@@ -66,9 +64,9 @@ class SelectOption():
         if value is None:
             value = ""
         elif value is not None and type(value) is not str:
-            raise InvalidArgument("label must be of type str, not " + str(type(value)))
+            raise WrongType("label", value, "str")
         elif value is not None and len(value) > 100 and value > 0:
-            raise InvalidArgument("label must be 100 or fewer and must be higher than 0 in length (" + str(len(value)) + ")")
+            raise InvalidLength("label", value, 100, 0)
         self._json["label"] = value
 
     @property
@@ -82,7 +80,7 @@ class SelectOption():
     @value.setter
     def value(self, value):
         if inspect.isclass(value):
-            raise Exception()
+            raise WrongType("value", value, ["int", "str", "bool", "float"])
         self._json["value"] = value
 
     @property
@@ -96,9 +94,9 @@ class SelectOption():
     @description.setter
     def description(self, value):
         if value is not MISSING and type(value) is not str:
-            raise InvalidArgument("description must be of type str, not " + " " + str(type(value)))
+            raise WrongType("description", "str")
         if value is not MISSING and len(value) > 50:
-            raise InvalidArgument("description must be 50 or fewer in length (" + str(len(value)) + ")")
+            raise InvalidLength("description", 50, 0)
         self._json["description"] = value
     
     @property
@@ -127,21 +125,23 @@ class SelectOption():
             return self._json["emoji"]["name"]
         return f'<{"a" if "animated" in self._json["emoji"] else ""}:{self._json["emoji"]["name"]}:{self._json["emoji"]["id"]}>'
     @emoji.setter
-    def emoji(self, val: Union[Emoji, str]):
+    def emoji(self, val: Union[Emoji, str, dict]):
         """The emoji appearing before the label"""
-        if type(emoji) is str:
+        if type(val) is str:
             self._json["emoji"] = {
                 "id": None,
-                "name": emoji
+                "name": val
             }
-        elif type(emoji) is Emoji:
+        elif type(val) is Emoji:
             self._json["emoji"] = {
-                "id": emoji.id,
-                "name": emoji.name,
-                "animated": emoji.animated
+                "id": val.id,
+                "name": val.name,
+                "animated": val.animated
             }
-        elif type(emoji) is dict:
-            self._json["emoji"] = emoji
+        elif type(val) is dict:
+            self._json["emoji"] = val
+        else:
+            raise WrongType("emoji", val, ["str", "discord.Emoji", "dict"])
 
 
     def to_dict(self) -> dict:
@@ -184,10 +184,6 @@ class SelectMenu():
         The position of the option that should be selected by default; default MISSING
     disabled: :class:`bool`, optional
         Whether the select menu should be disabled or not; default ``False``
-
-    Raises
-    ------
-    :raises: :class:`discord.InvalidArgument`: A passed argument was invalid
     """
     def __init__(self, custom_id, options, min_values = 1, max_values = 1, placeholder=MISSING, default=MISSING, disabled=False) -> None:
         """
@@ -198,50 +194,33 @@ class SelectMenu():
         SelectMenu(custom_id="my_id", options=[SelectOption(...)], min_values=2, placeholder="select something", default=0)
         ```
         """
-        if type(custom_id) is not str:
-            raise InvalidArgument("custom_id must be of type str, not " + str(type(custom_id)))
-        if len(custom_id) > 100:
-            raise InvalidArgument("custom_id must be 100 or fewer in length")
-        if len(custom_id) < 1:
-            raise InvalidArgument("custom_id cannot be empty")
         self._json = {
-            "type": ComponentType.SELECT_MENU,
-            "custom_id": custom_id,
-            "disabled": disabled
+            "type": ComponentType.SELECT_MENU
         }
+        self.custom_id = custom_id
+        self.disabled = disabled
+        self.options = options
 
-        if type(options) is list:
-            if len(options) > 25 or len(options) == 0:
-                raise InvalidArgument("You need to specify at least 1 option and at most 25 (" + str(len(options)) + ")")
-            if len(options) != [x for x in options if type(x) is SelectOption]:
-                self._json["options"] = [x.to_dict() for x in options]
-            elif len(options) != [x for x in options if type(x) is dict]:
-                self._json["options"] = options
-            else:
-                raise InvalidArgument("options must be of type SelectOption or dict, not " + str(type(options[0])))
-        else:
-            raise InvalidArgument("options must be of type List[SelectOption], not " + str(type(options)))
+        if placeholder is not MISSING:
+            self.placeholder = placeholder
 
         if min_values is not MISSING and max_values is MISSING:
             if min_values < 0 or min_values > 25:
-                raise InvalidArgument("min_values must be between 1 and 25")
-            self._json["min_values"] = min_values
-            self._json["max_values"] = min_values
+                raise OutOfValidRange("min_values", 1, 25)
+            self.min_values = min_values
+            self.max_values = min_values
         elif min_values is MISSING and max_values is not MISSING:
             if max_values < 0 or min_values > 25:
-                raise InvalidArgument("min_values must be between 1 and 25")
-            self._json["min_values"] = 1
-            self._json["max_values"] = max_values
+                raise OutOfValidRange("min_values", 1, 25)
+            self.min_values = 1
+            self.max_values = max_values
         elif min_values is not MISSING and max_values is not MISSING:
             if max_values < 0 or min_values > 25:
-                raise InvalidArgument("min_values must be between 1 and 25")
+                raise OutOfValidRange("min_values", 1, 25)
             if min_values < 0 or min_values > 25:
-                raise InvalidArgument("min_values must be between 1 and 25")
-            self._json["min_values"] = min_values
-            self._json["max_values"] = max_values
-
-        if placeholder is not MISSING:
-            self._json["placeholder"] = placeholder
+                raise OutOfValidRange("min_values", 1, 25)
+            self.min_values = min_values
+            self.max_values = max_values
         
         if default is not MISSING:
             self.set_default_option(default)
@@ -297,6 +276,10 @@ class SelectMenu():
         return self._json["custom_id"]
     @custom_id.setter
     def custom_id(self, value: str):
+        if len(value) > 100 or len(value) < 1:
+            raise InvalidLength("custom_id", 0, 100)
+        if type(value) is not str:
+            raise WrongType("custom_id", value, "str")
         self._json["custom_id"] = value
 
     @property
@@ -309,7 +292,17 @@ class SelectMenu():
         return [SelectOption._fromData(x) for x in self._json["options"]]
     @options.setter
     def options(self, value: List[SelectOption]):
-        self._json["options"] = [x.to_dict() for x in value]
+        if type(value) is list:
+            if len(value) > 25 or len(value) == 0:
+                raise OutOfValidRange("options", 1, 25)
+            if all(type(x) is SelectOption for x in value):
+                self._json["options"] = [x.to_dict() for x in value]
+            elif all(type(x) is dict for x in value):
+                self._json["options"] = value
+            else:
+                raise WrongType("options", value, ["List[SelectOption]", "List[dict]"])
+        else:
+            raise WrongType("options", value, "list")
 
     @property
     def default_option(self) -> Union[SelectOption, None]:
@@ -328,15 +321,11 @@ class SelectMenu():
         ----------
         position: :class:`int`
             The position of the option that should be default
-
-        Raises
-        ------
-        :raises: :class:`discord.InvalidArgument`: A passed argument was invalid
         """
         if type(position) is not int:
-            raise InvalidArgument("position has to be of type int, not " + str(type(position)))
-        if position < 0 or position > len(self.options):
-            raise InvalidArgument("default option position needs to be between 0 and " + str(len(self.options) - 1) + "(length of options)")
+            raise WrongType("position", position, "int")
+        if position < 0 or position >= len(self.options):
+            raise OutOfValidRange("default option position", 0, str(len(self.options) - 1))
         self._json["options"][position]["default"] = True
         return self
     
@@ -426,10 +415,6 @@ class Button():
         Whether a new line should be added before the button; default False
     disabled: :class:`bool`, optional
         Whether the button is disabled; default False
-
-    Raises
-    ------
-    :raises: :class:`discord.InvalidArgument`: A passed argument was invalid
     """
     def __init__(self, custom_id, label="\u200b", color = "blurple", emoji=MISSING, new_line=False, disabled=False) -> None:
         """
@@ -440,50 +425,17 @@ class Button():
         Button("my_custom_id", "This is a cool button", "green", new_line=True)
         ```
         """
+        self._json = {"type": ComponentType.BUTTON}
         if label is MISSING and emoji is MISSING:
             raise InvalidArgument("You need to pass a label or an emoji")
-        if label is not MISSING and type(label) is not str:
-            raise InvalidArgument("label must be of type str, not " + str(type(label)))
-        if type(custom_id) is not str:
-            raise InvalidArgument("custom_id must be of type str, not " + str(type(custom_id)))
-        if type(disabled) is not bool:
-            raise InvalidArgument("disabled must be of type bool") 
-        if emoji is not MISSING and type(emoji) not in [Emoji, str, dict]:
-            raise InvalidArgument("emoji must be of type discord.Emoji or str, not " + str(type(emoji)))
-        if len(custom_id) > 100:
-            raise InvalidArgument("custom_id maximum character limit (100) exceeded")
-        if len(custom_id) < 1:
-            raise InvalidArgument("custom_id must be longer than 0 characters")
-        if label is not MISSING and len(label) > 80:
-            raise InvalidArgument("lavel maximum character limit (80) exceeded")
-        if label is not MISSING and len(label) < 1:
-            raise InvalidArgument("label must be longer than 0 characters")
-        if Colors.getColor(color) is None:
-            raise InvalidArgument(str(color) + " is not a valid color")
-
+        
+        self.custom_id = custom_id
         self.new_line = new_line
-        self._json = {
-            "type": ComponentType.BUTTON,
-            "custom_id": custom_id,
-            "style": Colors.getColor(color),
-            "disabled": bool(disabled),
-        }
-        if label is not MISSING:
-            self._json["label"] = label
+        self.label = label
+        self.color = color
         if emoji is not MISSING:
-            if type(emoji) is str:
-                self._json["emoji"] = {
-                    "id": None,
-                    "name": emoji
-                }
-            elif type(emoji) is Emoji:
-                self._json["emoji"] = {
-                    "id": emoji.id,
-                    "name": emoji.name,
-                    "animated": emoji.animated
-                }
-            elif type(emoji) is dict:
-                self._json["emoji"] = emoji
+            self.emoji = emoji
+        self.disabled = disabled
 
     def to_dict(self):
         return self._json
@@ -521,12 +473,11 @@ class Button():
     @custom_id.setter
     def custom_id(self, val: str):
         if type(val) is not str:
-            raise InvalidArgument("custom_id must be of type str, not " + str(type(val)))
-        
+            raise WrongType("custom_id", val, "str")
         if len(val) > 100:
-            raise InvalidArgument("custom_id must be shorter than 100 characters")
+            raise InvalidLength("custom_id", _max=100)
         if len(val) < 1:
-            raise InvalidArgument("custom_id must be longer than 0 characters")
+            raise InvalidLength("custom_id", _min=0)
 
         self._json["custom_id"] = str(val)
 
@@ -543,11 +494,11 @@ class Button():
         if val is None:
             val = ""
         elif val is not None and type(val) is not str:
-            raise InvalidArgument("label must be of type str, not " + str(type(val)))
+            raise WrongType("label", val, "str")
         elif val is not None and len(val) > 80:
-            raise InvalidArgument("label must be shorter than 80 characters")
+            raise InvalidLength("label", _max=80)
         elif val is not None and len(val) < 1:
-            raise InvalidArgument("label must be longer than 0 characters")
+            raise InvalidLength("label", _min=0)
 
         self._json["label"] = str(val)
 
@@ -580,9 +531,7 @@ class Button():
             return self._json["emoji"]["name"]
         return f'<{"a" if "animated" in self._json["emoji"] else ""}:{self._json["emoji"]["name"]}:{self._json["emoji"]["id"]}>'
     @emoji.setter
-    def emoji(self, val: Union[Emoji, str]):
-        if type(val) not in [Emoji, str]:
-            raise InvalidArgument("emoji msut be of type discord.Emoji or str, not " + str(type(val)))
+    def emoji(self, val: Union[Emoji, str, dict]):
         if type(val) is str:
             self._json["emoji"] = {
                 "id": None,
@@ -594,6 +543,10 @@ class Button():
                 "name": val.name,
                 "animated": val.animated
             }
+        elif type(val) is dict:
+            self._json["emoji"] = val
+        else:
+            raise WrongType("emoji", val, ["str", "discord.Emoji", "dict"])
 
     @property
     def disabled(self) -> bool:
@@ -606,12 +559,9 @@ class Button():
         return self._json["disabled"] if "disabled" in self._json else False
     @disabled.setter
     def disabled(self, val):
-        if type(val) != bool:
-            raise InvalidArgument("disabled must be type of bool")
-        if "disabled" in self._json:
-            self._json["disabled"] = bool(val)
-        else:
-            self._json |= {"disabled": bool(val)}
+        if type(val) is not bool:
+            raise WrongType("disabled", val, "bool")
+        self._json["disabled"] = bool(val)
     
     @property
     def hash(self) -> str:
@@ -657,11 +607,6 @@ class LinkButton():
         Whether a new line should be added before the button; default False
     disabled: :class:`bool`, optional
         Whether the button is disabled; default False
-
-    
-    Raises
-    ------
-    :raises: :class:`discord.InvalidArgument`: A passed argument was invalid
     """
     def __init__(self, url, label="\u200b", emoji=MISSING, new_line=False, disabled=False) -> None:
         """
@@ -669,51 +614,21 @@ class LinkButton():
         
         Example:
         ```py
-        LinkButton("https://discord.com", "press me (if you can)!", emoji="😀", disabled=True)
+        LinkButton("https://discord.com/", "press me (if you can)!", emoji="😀", disabled=True)
         ```
         """
+        self._json = {"type": ComponentType.BUTTON, "style": Colors.url}
         if label is MISSING and emoji is MISSING:
-            raise InvalidArgument("You need to pass label or emoji")
-        if label is not MISSING and type(label) is MISSING:
-            raise InvalidArgument("label must be of type str, not " + str(type(label)))
-        if label is not MISSING and type(label) is not str:
-            raise InvalidArgument("label must be of type str, not " + str(type(label)))
-        if type(url) is not str:
-            raise InvalidArgument("url must be of type str, not " + str(type(url)))
-        if not url.startswith("http://") and not url.startswith("https://"):
-            raise InvalidArgument("Link must start with 'http://' or 'https://'")
-        if type(disabled) is not bool:
-            raise InvalidArgument("disabled must be of type bool, not " + str(type(disabled)))
-        if emoji is not MISSING and type(emoji) not in [Emoji, str, dict]:
-            raise InvalidArgument("emoji msut be of type discord.Emoji or str, not " + str(type(emoji)))
-        if label is not MISSING and len(label) > 80:
-            raise InvalidArgument("lavel maximum character limit (80) exceeded")
-        if label is not MISSING and len(label) < 1:
-            raise InvalidArgument("label must be longer than 0 characters")
+            raise InvalidArgument("You need to pass a label or an emoji")
 
+        self.label = label
+        self.url = url
         self.new_line = new_line
-        self._json = {
-            "type": ComponentType.BUTTON,
-            "url": url,
-            "style": 5,
-            "disabled": disabled
-        }
-        if label is not MISSING:
-            self._json["label"] = label
+        self.disabled = disabled
+
         if emoji is not MISSING:
-            if type(emoji) is str:
-                self._json["emoji"] = {
-                    "id": None,
-                    "name": emoji
-                }
-            elif type(emoji) is Emoji:
-                self._json["emoji"] = {
-                    "id": emoji.id,
-                    "name": emoji.name,
-                    "animated": emoji.animated
-                }
-            elif type(emoji) is dict:
-                self._json["emoji"] = emoji
+            self.emoji = emoji
+
 
     def to_dict(self):
         return self._json
@@ -721,10 +636,11 @@ class LinkButton():
     # region props
     @property
     def component_type(self) -> int:
-        """The message component type for this component
+        """
+        The message component type
 
             .. note::
-                The component type will always be 2 (button)
+                The message component type will be always 2 (button)
         
         :type: :class:`int`
         """
@@ -733,7 +649,7 @@ class LinkButton():
     @property
     def content(self) -> str:
         """
-        The whole content of the button text, consiting of the emoji and the label
+        The complete content in the button ("{emoji} {label}")
 
         :type: :class:`str`
         """
@@ -742,40 +658,54 @@ class LinkButton():
     @property
     def url(self) -> str:
         """
-        The url which will be opened after the button is pressed
+        The link which will be opened upon pressing the button
 
-        :type: class:`str`
+        :type: :class:`str`
         """
         return self._json["url"]
     @url.setter
     def url(self, val: str):
         if type(val) is not str:
-            raise InvalidArgument("url must be of type str, not " + str(type(val)))
-        if not val.startswith("http://") and not val.startswith("https://"):
-            raise InvalidArgument("Link must start with 'https://' or 'http://'")
-        self._json["url"] = val
-    
+            raise WrongType("url", val, "str")
+        self._json["url"] = str(val)
+
     @property
     def label(self) -> str:
         """
-        The text that appears on the button
+        The label displayed on the button
 
         :type: :class:`str`
         """
         return self._json.get("label", None)
     @label.setter
-    def label(self, val):
-        if type(val) is not str:
-            raise InvalidArgument("label must be of type str, not " + str(type(val)))
-        self._json["label"] = val 
+    def label(self, val: str):
+        if val is None:
+            val = ""
+        elif val is not None and type(val) is not str:
+            raise WrongType("label", val, "str")
+        elif val is not None and len(val) > 80:
+            raise InvalidLength("label", _max=80)
+        elif val is not None and len(val) < 1:
+            raise InvalidArgument("label", _min=0)
 
+        self._json["label"] = str(val)
+
+    @property
+    def color(self) -> int:
+        """
+        The color for the button, will always be ``5`` (link)
+
+        :type: :class:`int`
+        """
+        return self._json["style"]
+    
     @property
     def emoji(self) -> str:
         """The mention of the emoji before the text
-
+        
             .. note::
-                For setting the emoji you can use `str` or `discord.Emoji`
-
+                For setting the emoji, you can use a str or discord.Emoji          
+        
         :type: :class:`str`
         """
         if "emoji" not in self._json:
@@ -784,9 +714,7 @@ class LinkButton():
             return self._json["emoji"]["name"]
         return f'<{"a" if "animated" in self._json["emoji"] else ""}:{self._json["emoji"]["name"]}:{self._json["emoji"]["id"]}>'
     @emoji.setter
-    def emoji(self, val: Union[Emoji, str]):
-        if type(val) not in [Emoji, str]:
-            raise InvalidArgument("emoji msut be of type discord.Emoji or str, not " + str(type(val)))
+    def emoji(self, val: Union[Emoji, str, dict]):
         if type(val) is str:
             self._json["emoji"] = {
                 "id": None,
@@ -798,30 +726,25 @@ class LinkButton():
                 "name": val.name,
                 "animated": val.animated
             }
-
-    @property
-    def color(self) -> int:
-        """
-        The color sttyle for the button
-        
-            .. note::
-                The color style Will always be 5 (link_button)
-        
-        :type: :class:`int`
-        """
-        return self._json["style"]
+        elif type(val) is dict:
+            self._json["emoji"] = val
+        else:
+            raise WrongType("emoji", val, ["str", "discord.Emoji", "dict"])
 
     @property
     def disabled(self) -> bool:
         """
-        Whether the button is disabled or not
-        
+        Whether the button is dissabled
+        If true, the button is shown but not clickable
+
         :type: :class:`bool`
         """
-        return self._json["disabled"]
+        return self._json["disabled"] if "disabled" in self._json else False
     @disabled.setter
     def disabled(self, val):
-        self._json["disabled"] = val
+        if type(val) is not bool:
+            raise WrongType("disabled", val, "bool")
+        self._json["disabled"] = bool(val)
     # endregion
 
     @classmethod
@@ -845,7 +768,7 @@ class Colors:
     Secondary   =   grey            = 2
     Succes      =   green           = 3
     Danger      =   red             = 4
-    URL                             = 5
+    url                             = 5
 
     @classmethod
     def getColor(cls, s):
